package fr.smallcrew.security.service;

import fr.smallcrew.security.domain.UserToken;
import fr.smallcrew.security.repository.UserTokenRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.web.authentication.rememberme.PersistentRememberMeToken;
import org.springframework.security.web.authentication.rememberme.PersistentTokenRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
@Transactional
public class UserTokenService implements PersistentTokenRepository {
  @Autowired
  private UserTokenRepository userTokenRepository;

  @Override
  public void createNewToken(PersistentRememberMeToken token) {
    UserToken userToken = new UserToken();
    userToken.setDate(token.getDate());
    userToken.setSeries(token.getSeries());
    userToken.setTokenValue(token.getTokenValue());
    userToken.setUsername(token.getUsername());
    userTokenRepository.save(userToken);
  }

  @Override
  public void updateToken(String series, String tokenValue, Date lastUsed) {
    UserToken token = userTokenRepository.findBySeries(series);
    token.setSeries(series);
    token.setTokenValue(tokenValue);
    token.setDate(lastUsed);
    userTokenRepository.save(token);
  }

  @Override
  public PersistentRememberMeToken getTokenForSeries(String seriesId) {
    UserToken token = userTokenRepository.findBySeries(seriesId);
    PersistentRememberMeToken t = null;
    if (token != null) {
      t = new PersistentRememberMeToken(token.getUsername(), token.getSeries(), token.getTokenValue(), token.getDate());
    }
    return t;
  }

  @Override
  public void removeUserTokens(String username) {
    List<UserToken> userTokens = userTokenRepository.findByUsername(username);
    for (UserToken userToken : userTokens) {
      userTokenRepository.delete(userToken);
    }
  }
}
